<?php

class Wptc_Revision_Limit extends WPTC_Privileges {
	protected $config;

	public function __construct() {
		$this->config = WPTC_Pro_Factory::get('Wptc_Revision_Limit_Config');
	}

	public function init() {
		if ($this->is_privileged_feature(get_class($this)) && $this->is_switch_on()) {
			$supposed_hooks_class = get_class($this) . '_Hooks';
			WPTC_Pro_Factory::get($supposed_hooks_class)->register_hooks();

			$this->update_revision_limit();
		}
	}

	private function is_switch_on(){
		return true;
	}

	private function update_revision_limit(){
		$args = $this->config->get_option('privileges_args');

		if(!empty($args)){
			$args = json_decode($args, true);
			$this_class_name = get_class($this);
			$revision_obj = $args[$this_class_name];
			$revision_days = $revision_obj['days'];
		}

		if(empty($revision_days)){
			$revision_days = WPTC_FALLBACK_REVISION_LIMIT_DAYS;
		}

		$this->validate_revision_limit($revision_days);

		$this->config->set_option('revision_limit', $revision_days);
	}

	private function validate_revision_limit(&$revision_days){
		// dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		if ($revision_days <= WPTC_DEFAULT_MAX_REVISION_LIMIT) {
			dark_debug(array(), '--------RETURNEING--------');
			return ;
		}
		$default_repo = $this->config->get_option('default_repo');
		$cloud_repo = WPTC_Factory::get($default_repo);

		if (empty($cloud_repo)) {
			$revision_days = WPTC_DEFAULT_MAX_REVISION_LIMIT;
			return ;
		}

		$is_authorized = $cloud_repo->is_authorized(true);

		if (empty($is_authorized)) {
			$revision_days = WPTC_DEFAULT_MAX_REVISION_LIMIT;
			return ;
		}


		switch ($default_repo) {
			case 'g_drive':
				$revision_days = WPTC_DEFAULT_MAX_REVISION_LIMIT;
				break;
			case 'dropbox':
				$revision_days = $cloud_repo->validate_max_revision_limit();
				break;
			case 's3':
				$revision_days = $cloud_repo->validate_max_revision_limit();
				break;
		}
		// dark_debug($revision_days, '--------$revision_days--------');
	}

}