<?php

class Wptc_Staging extends WPTC_Privileges {

	const CLONE_TMP_FOLDER = 'wptc_staging_controller';

	protected $processed_files;
	protected $staging_id;
	protected $db_backup;
	protected $options;
	protected $logger;
	protected $config;
	protected $filesystem;
	protected $plugin_bridge_path;
	protected $same_staging_folder;
	protected $same_staging_bridge_dir;
	protected $processed_db;
	protected $exclude_class_obj;
	protected $staging_common;
	protected $file_iterator;
	protected $app_functions;

	public function __construct(){
		$this->options = WPTC_Pro_Factory::get('Wptc_staging_Config');
		$this->db_backup = WPTC_Factory::get('databaseBackup');
		$this->processed_files = WPTC_Factory::get('processed-files');
		$this->exclude_class_obj = WPTC_Base_Factory::get('Wptc_ExcludeOption');
		$this->app_functions = WPTC_Base_Factory::get('Wptc_App_Functions');
		$this->processed_db = new WPTC_Processed_iterator();
		$this->staging_common = new WPTC_Stage_Common();
		$this->file_iterator = new WPTC_File_Iterator();
		$this->init_db();
		$this->logger = WPTC_Factory::get('logger');
		$this->config = WPTC_Factory::get('config');
		$this->plugin_bridge_path = dirname(__FILE__) . '/' . 'bridge'.'/';
		$this->init_staging_id();
		$this->run_updates();
	}

	public function init() {
		if ($this->is_privileged_feature(get_class($this)) && $this->is_switch_on()) {
			$supposed_hooks_class = get_class($this) . '_Hooks';
			WPTC_Pro_Factory::get($supposed_hooks_class)->register_hooks();
		}
	}

	private function is_switch_on(){
		return true;
	}

	private function init_fs(){
		$this->filesystem = $this->staging_common->init_fs();
	}

	private function init_db(){
		$this->wpdb = $this->staging_common->init_db();
	}

	private function init_staging_id(){
		$this->staging_id = $this->staging_common->init_staging_id();
	}

	//Process staging feature updates for existing users.
	public function run_updates(){
		$updates = $this->config->get_option('run_staging_updates');
		if (empty($updates)) {
			return false;
		}
		if (version_compare('1.9.0', $updates) >= 0) {
			$prefix = $this->get_existing_staging_prefix();
			if ($prefix !== false && $prefix !== $this->wpdb->base_prefix) {
				$this->discourage_search_engine($prefix, $reset_permalink = true);
			}
		}
		$this->config->set_option('run_staging_updates', false);
	}

	public function get_staging_details($param = null){
		$serialized_details = $this->options->get_option('same_server_staging_details');
		if (empty($serialized_details)) {
			return false;
		}

		$unserialized_details = unserialize($serialized_details);
		if (empty($unserialized_details)) {
			return false;
		}
		if (empty($param)) {
			return $unserialized_details;
		}
		return $unserialized_details[$param];
	}

	public function is_any_staging_process_going_on(){
		if (empty($this->options)) {
			$this->options = WPTC_Pro_Factory::get('Wptc_staging_Config');
		}
		if ($this->options->get_option('is_staging_running', true) || $this->options->get_option('same_server_staging_running', true) ) {
			return true;
		}
		return false;
	}

	private function internal_staging_delete_bridge(){
		dark_debug('Function :','---------'.__FUNCTION__.'-----------------');
		dark_debug($this->same_staging_bridge_dir, '------$this->same_staging_bridge_dir---------------');
		return $this->filesystem->delete($this->same_staging_bridge_dir, true);
	}

	public function delete_staging_wptc(){
		$this->init_fs();
		$this->delete_internal_staging();
	}

	public function stop_staging_wptc(){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		$this->same_server_update_keys('same_server_staging_status', 'stop_staging');
	}

	private function delete_internal_staging($dont_print = false){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		$db = $this->delete_internal_staging_db();
		$files = $this->delete_internal_staging_files();
		$flags = $this->delete_internal_staging_flags();
		dark_debug($db, '---------$db------------');
		dark_debug($files, '---------$files------------');
		if ($dont_print) {
			return false;
		}
		if ($db && $files) {
			die_with_json_encode(array('status' => 'success', 'deleted' => 'both'));
		} else if ($db) {
			die_with_json_encode(array('status' => 'success', 'deleted' => 'db'));
		} else if($files){
			die_with_json_encode(array('status' => 'success', 'deleted' => 'files'));
		} else {
			die_with_json_encode(array('status' => 'error', 'deleted' => 'none'));
		}
	}

	public function delete_internal_staging_db($db_prefix = false){
		$db_prefix = $this->get_staging_details('db_prefix');
		dark_debug($db_prefix, '--------$db_prefix 1--------');

		if (empty($db_prefix)) {
			$db_prefix =  $this->options->get_option('same_server_staging_db_prefix');
		}

		dark_debug($db_prefix, '---------------$db_prefix-----------------');

		if (empty($db_prefix)) {
			return false;
		}

		if ($db_prefix == $this->wpdb->base_prefix) {
			return false;
		}

		return $this->processed_files->drop_tables_with_prefix($db_prefix);
	}

	public function delete_internal_staging_files($hard_delete = false){
		$staging_folder = $this->get_staging_details('staging_folder');
		if (empty($staging_folder)) {
			$staging_folder = $this->options->get_option('same_server_staging_path');
		}
		dark_debug($staging_folder, '---------------$staging_folder global-----------------');

		if (empty($staging_folder)) {
			return false;
		}
		$staging_path = get_home_path_wptc() . $staging_folder . '/';
		$staging_path = wp_normalize_path($staging_path);

		//check its live site before deleting it
		if (trailingslashit(wp_normalize_path(get_home_path_wptc())) == trailingslashit($staging_path) || trailingslashit(WPTC_ABSPATH) == trailingslashit($staging_path)) {
			return false;
		}

		dark_debug($staging_path, '---------$staging_path------------');
		return $this->filesystem->delete($staging_path, true);
	}

	private function delete_internal_staging_flags(){
		$this->options->set_option('staging_type', false);
		$this->options->set_option('same_server_staging_details', false);
		$this->options->set_option('same_server_staging_status', false);
		$this->options->set_option('is_staging_completed', false);
		$this->options->set_option('staging_id', false);
		$this->hard_reset_staging_flags();
		$this->options->flush();
	}

	private function hard_reset_staging_flags(){
		$this->options->set_option('staging_type', false);
		$this->options->set_option('is_staging_completed', false);
		$this->options->set_option('staging_completed', false);
		$this->options->set_option('staging_id', false);
	}

	public function get_staging_url_wptc(){
		dark_debug(array(), '-----------get_staging_url_wptc----------');
		$destination_url = $this->same_server_staging_url();
		die_with_json_encode(array('destination_url' => $destination_url), 1);
	}

	public function test_connection($path = false) {
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		$this->options->set_option('staging_type', 'internal');
		$this->options->flush();
		$this->processed_db->truncate();
		$this->init_staging_id();
		$this->options->set_option('same_server_copy_staging', true);
		do_action('send_report_data_wptc', $this->staging_id, 'LIVE_TO_STAGING', 'STARTED');
		$this->same_server_copy_bridge_files($path);
	}

	public function choose_action($path = false, $reqeust_type = false){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");

		$this->options->set_option('last_staging_ping', time());

		$this->app_functions->set_start_time();

		$this->init_fs();

		$this->same_server_set_staging_path();

		$status = $this->options->get_option('same_server_staging_status', true);

		if (empty($status) || $reqeust_type === 'copy' || $reqeust_type === 'fresh') {
			$status = 'test_connection';
		}

		dark_debug($status, '--------$current_action--------');

		if ($status === 'test_connection') {
			$this->test_connection($path);
		}

		if ($status === 'stop_staging') {
			do_action('send_report_data_wptc', $this->staging_id, 'LIVE_TO_STAGING', 'CANCELLED');
			$this->process_stop_staging_req();
		}

		if ($status === 'test_bridge_over' || $status === 'db_clone_progress') {
			$this->process_db_clone_req($status);
		}

		if($status === 'db_clone_over' || $status === 'copy_files_progress'){
			$this->process_copy_files_req($status);
		}

		if($status === 'copy_files_over'){
			$this->process_replace_links_req($status);
		}

		if($status === 'replace_links_over'){
			$this->process_staging_completed_req();
		}

		$this->staging_completed_email($type = 'internal');

		die_with_json_encode(array('status' => 'success', 'msg' => 'Staging is completed.', 'percentage' => 100));
	}

	private function process_stop_staging_req(){
		$this->delete_internal_staging($dont_print = true);
		die_with_json_encode(array('status' => 'success', 'msg' => 'Stopping Staging process.', 'percentage' => 0));
	}

	private function process_db_clone_req($status){
		if ($status === 'test_bridge_over') {
			$this->logger->log("Starting to clone tables.", 'staging_progress', $this->staging_id);
		}

		$this->do_copy_staging_actions();

		$staging_path = $this->options->get_option('same_server_staging_path');

		if (!empty($staging_path)) {
			$this->same_server_copy_bridge_files($staging_path, true);
		}

		$this->same_server_update_keys('same_server_staging_status', 'db_clone_progress');
		$this->same_server_clone_db();
		$this->same_server_update_keys('same_server_staging_status', 'db_clone_over');
		$this->logger->log("DB has been cloned successfully", 'staging_progress', $this->staging_id);

		die_with_json_encode(array('status' => 'continue', 'msg' => 'Database is cloned.', 'percentage' => 30));
	}

	private function process_copy_files_req($status){
		if ($status === 'db_clone_over') {
			$this->logger->log("Copying Files is started.", 'staging_progress', $this->staging_id);
		}

		$this->same_server_update_keys('same_server_staging_status', 'copy_files_progress');
		$iter_count = $this->same_server_copy();

		$this->same_server_update_keys('same_server_staging_status', 'copy_files_over');

		$this->logger->log("All files are copied to staging location succesfully.", 'staging_progress', $this->staging_id);

		die_with_json_encode(array('status' => 'continue', 'msg' => 'Files are copied successfully' , 'percentage' => 75));
	}

	private function process_replace_links_req($status){
		$this->same_server_replace_links();
		$this->same_server_update_keys('same_server_staging_status', 'replace_links_over');
		$this->logger->log("Replaced links in the staging site successfully", 'staging_progress', $this->staging_id);

		die_with_json_encode(array('status' => 'continue', 'msg' => 'Links are replaced.', 'percentage' => 90));
	}

	private function process_staging_completed_req(){
		$this->rewrite_permalink_structure();
		WPTC_Base_Factory::get('WPTC_Update_In_Staging')->update_in_staging();
		$this->internal_staging_delete_bridge();
		$this->complete();
		$this->processed_db->truncate();

		$this->logger->log("Site staged succesfully.", 'staging_progress', $this->staging_id);

		do_action('send_report_data_wptc', $this->staging_id, 'LIVE_TO_STAGING', 'SUCCESS');

		$this->same_server_update_keys('same_server_staging_status', 'staging_completed');
		$staging_path = $this->options->get_option('same_server_staging_path');

		if (empty($staging_path)) {
			return false;
		}

		$db_prefix = $this->get_staging_db_prefix('full_prefix');

		$this->options->set_option('same_server_staging_details',
						serialize(
							array(
								'destination_url' => get_home_url().'/'.$staging_path.'/',
								'human_completed_time' => user_formatted_time_wptc(time()),
								'timestamp' => time(),
								'db_prefix' => $db_prefix,
								'staging_folder' => $staging_path
								)
							)
						);

		$this->options->flush();
	}

	private function staging_completed_email($type = false){
		$email_data = array(
			'type' => 'staging_completed',
			'staging_url' => $this->get_staging_details('destination_url'),
		);
		error_alert_wptc_server($email_data);
	}

	private function do_copy_staging_actions(){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		if(!$this->options->get_option('same_server_copy_staging')){
			dark_debug(array(), '--------Get out--------');
			return false;
		}
		$db = $this->delete_internal_staging_db();
		dark_debug($db, '--------$db--------');
		$files = $this->delete_internal_staging_files();
		dark_debug($files, '--------$files--------');
		$this->options->set_option('same_server_staging_details', false);
	}

	private function same_server_update_keys($key, $value){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");

		if ($key === 'same_server_staging_status') {
			$current_value = $this->config->get_option('same_server_staging_status', true);
			dark_debug($current_value, '--------$current_value--------');
			if ($current_value == "stop_staging") {
				dark_debug(array(), '--------RETURNN--------');
				return false;
			}
		}

		$this->config->set_option($key, $value);
		$updated_result = $this->config->get_option($key, true);
		dark_debug($updated_result, '--------$updated_result--------');
	}

	private function is_same_server_running(){
		return $this->options->get_option('same_server_staging_running', true);
	}

	private function rewrite_permalink_structure(){

		$new_prefix = $this->get_staging_db_prefix('full_prefix');

		if (empty($new_prefix) || $new_prefix == $this->wpdb->base_prefix) {
			return false;
		}

		$result = $this->wpdb->query(
			"UPDATE " . $new_prefix . "options SET option_value=''  WHERE option_name = 'permalink_structure'"
		);

		if (empty($result)) {
			return $this->logger->log("rewrite permalink structure failed.", 'staging_progress', $this->staging_id);
		} else {
			return $this->logger->log("rewrite permalink structure successfully done.", 'staging_progress', $this->staging_id);
		}

	}

	private function same_server_copy_bridge_files($path = false, $do_not_die = false){
		if ($path === false) {
			$path = $this->get_staging_details('staging_folder');
		}
		dark_debug($path, '---------$path same_server_copy_bridge_files------------');
		$path = wp_normalize_path($path);
		$this->same_server_set_staging_path($path);
		$this->same_server_update_keys('same_server_staging_status', 'test_bridge');
		$this->app_functions->mkdir_by_path($this->same_staging_bridge_dir);
		if ($this->filesystem->exists($this->same_staging_bridge_dir) !== true && $do_not_die === false) {
			$this->same_server_update_keys('same_server_staging_status', 'test_bridge_folder_create_failed');
			$this->logger->log("Testing connection by uploading bridge file is failed", 'staging_failed', $this->staging_id);
			die_with_json_encode(array('status' => 'error' , 'msg' => 'Cannot create staging folder.'));
		}

		$cp_result = $this->filesystem->copy($this->plugin_bridge_path . 'bridge.php', $this->same_staging_bridge_dir . 'bridge.php', true, FS_CHMOD_FILE);
		if(!$cp_result && $do_not_die === false){
			$this->same_server_update_keys('same_server_staging_status', 'test_bridge_file_copy_failed');
			$this->logger->log("Testing connection by uploading bridge file is failed", 'staging_failed', $this->staging_id);
			die_with_json_encode(array('status' => 'error' , 'msg' => 'Cannot create copy bridge file.'));
		}

		$this->same_server_update_keys('same_server_staging_status', 'test_bridge_over');
		$this->options->set_option('same_server_staging_path', $path);
		$this->options->set_option('same_server_staging_running', true);

		if ($do_not_die === false) {
			$this->logger->log("Tested connection succesfully.", 'staging_progress', $this->staging_id);
			die_with_json_encode(array('status' => 'continue', 'msg' => 'Test bridge copied.', 'percentage' => 10));
		}
	}

	private function skip_file($file){
		if(!is_readable($file)){
			return true;
		}

		if (stripos($file . '/' , $this->same_staging_folder) !== false || is_wptc_file($file)) {
			return true;
		}

		if ($this->exclude_class_obj->is_excluded_file($file)) {
			return true;
		}

		return false;
	}

	private function same_server_copy(){
		$this->get_folders();
		$this->copy_selected_folders();
	}

	private function get_folders(){
		$get_folders = $this->config->get_option('same_server_get_folders');

		if (!empty($get_folders)) {
			return false;
		}

		$this->file_iterator->get_folders();
		$this->config->set_option('same_server_get_folders', true);
	}

	private function copy_selected_folders(){

		$break = false;

		while(!$break){
			$dir_meta = $this->processed_db->get_unfnished_folder();

			dark_debug($dir_meta, '--------$dir_meta--------');

			$deep_dirs = false;

			if (empty($dir_meta) || $dir_meta->count === -1) {
				$break = true;
				dark_debug(array(), '--------SKIP META 1--------');
				continue;
			}

			if( array_search($dir_meta->name, $this->file_iterator->get_deep_dirs()) !== false ){
				$deep_dirs = true;
			}

			if ($deep_dirs === false && $this->skip_file($dir_meta->name) === true) {
				dark_debug(array(), '--------SKIP FILE OR FOLDER 2--------');
				$this->processed_db->update_iterator($dir_meta->name, -1);
				continue;
			}

			if(wptc_is_dir($dir_meta->name)){
				dark_debug(array(), '--------COPY DIR--------');
				$this->copy_dir($dir_meta->name, $dir_meta->count, $deep_dirs);
			} else {
				dark_debug(array(), '--------COPY FILE--------');
				$this->copy_file($dir_meta->name, $update_status = true);
			}
		}
	}

	private function copy_file($live_file, $update_status = false){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");

		$live_file = wp_normalize_path($live_file);


		$staging_file = $this->same_server_replace_pathname($live_file);

		$this->app_functions->mkdir_by_path(dirname($staging_file));

		$size = filesize($live_file);

		if ($size > WPTC_STAGING_COPY_SIZE) {
			dark_debug(array(), '--------large file--------');
			$copy_status = wptc_copy_large_file($live_file, $staging_file);
		} else {
			$copy_status = $this->filesystem->copy($live_file, $staging_file, true, FS_CHMOD_FILE);
		}

		dark_debug(array(), '--------Copy status--------');

		if (!$copy_status) {
			$this->logger->log('Could not copy this file - '.$live_file, 'staging_progress', $this->staging_id);
		}

		if ($update_status) {
			$this->processed_db->update_iterator($live_file, -1);
		}
	}

	private function copy_dir($live_path, $offset, $deep_dirs){

		if ($deep_dirs) {
			$files_obj = $this->file_iterator->get_files_obj_by_path($live_path, $recursive = false);
		} else {
			$files_obj = $this->file_iterator->get_files_obj_by_path($live_path, $recursive = true);
		}

		$counter = $temp_counter = 0;

		$timeout_limit = $this->options->get_option('internal_staging_file_copy_limit');
		$timeout_limit = empty($timeout_limit) ? WPTC_STAGING_DEFAULT_FILE_COPY_LIMIT : $timeout_limit;

		foreach ($files_obj as $file_obj) {
			if ($counter++ < $offset ) {
				continue;
			}

			$offset++;

			$file = $file_obj->getPathname();

			$file = wp_normalize_path($file);

			if ($deep_dirs && wptc_is_dir($file)){
				dark_debug($file, '--------skip because of deep dir--------');
				continue;
			}

			dark_debug($file, '--------$file--------');

			if ($this->skip_file($file) === true) {
				dark_debug(array(), '--------SKIP FILE FOLDERS 3--------');
				$this->check_timeout_iter_file($live_path, $offset, $temp_counter, $timeout_limit);
				continue;
			}


			if(wptc_is_dir($file)){
				continue;
			}

			$this->copy_file($file);

			$this->check_timeout_iter_file($live_path, $offset, $temp_counter, $timeout_limit);
		}
		dark_debug(array(), '--------UPDATE FOLDER--------');
		$this->processed_db->update_iterator($live_path, -1);
	}

	private function check_timeout_iter_file($path, $offset, &$temp_counter, $timeout_limit){

		if (++$temp_counter < $timeout_limit) {
			return false;
		}

		$this->processed_db->update_iterator($path, $offset);

		die_with_json_encode(array('status' => 'continue', 'msg' => $this->app_functions->get_processing_files_count($type = 'internal_staging'), 'percentage' => 50));
	}

	private function complete(){
		$this->wpdb->query("TRUNCATE TABLE `".$this->wpdb->base_prefix."wptc_current_process`;");
		$this->wpdb->query("TRUNCATE TABLE `".$this->wpdb->base_prefix."wptc_processed_iterator`;");
	}

	public function is_staging_need_request(){

		if(!$this->is_any_staging_process_going_on()){
			die_with_json_encode(array('status' => false, 'check_again' => false));
		}

		$last_staging_ping = $this->options->get_option('last_staging_ping');

		if (empty($last_staging_ping)) {
			die_with_json_encode(array('status' => false, 'check_again' => false));
		}

		if ($last_staging_ping + ( WPTC_TIMEOUT + WPTC_TIMEOUT)  <  time() )  {
			die_with_json_encode(array('status' => true, 'check_again' => false));
		}

		die_with_json_encode(array('status' => false, 'check_again' => true));
	}

	private function same_server_add_completed_table(){
		$completed_tables = $this->options->get_option('same_server_clone_db_completed_tables');
		if (empty($completed_tables)) {
			return $this->options->set_option('same_server_clone_db_completed_tables', 1);
		}
		return $this->options->set_option('same_server_clone_db_completed_tables', $completed_tables + 1);
	}

	private function same_server_clone_db() {
		$limit = $this->options->get_option('internal_staging_db_rows_copy_limit');
		if (empty($limit)) {
			$limit = 1000; //fallback to default value
		}
		$wp_tables = $this->processed_files->get_all_tables();
		$this->options->set_option('same_server_clone_db_total_tables', count($wp_tables));
		dark_debug($wp_tables, '---------$wp_tables------------');
		foreach ($wp_tables as $table) {
			if (!$this->is_same_server_running()) {
				die_with_json_encode(array('status' => 'success', 'msg' => 'Staging process stopped.', 'percentage' => 0));
			}
			dark_debug($table, '---------$table------------');

			$unique_prefix = $this->get_staging_db_prefix();
			$new_table =  $unique_prefix . '_' . $table;

			dark_debug($new_table, '---------$new_table------------');
			$unique_prefix = (string) $unique_prefix;

			if ($this->exclude_class_obj->is_excluded_table($table) || stripos($table, $unique_prefix) !== false) {
				dark_debug($table, '---------------$table excluded from staging-----------------');
				$this->same_server_add_completed_table();
				continue;
			}

			dark_debug($new_table, '---------$new_table------------');
			if (!$this->processed_db->is_complete($table)) {
				if (is_wptc_table($table)) {
					$this->processed_db->update_iterator($table, -1); //Done
					dark_debug(array(), '---------Done already so skipping------------');
					$this->same_server_add_completed_table();
					continue;
				}

				$table_meta = $this->staging_common->get_table_data($table);
				extract($table_meta);
			} else {
				dark_debug(array(), '--------skiipped---------');
				$this->same_server_add_completed_table();
				continue;
			}
			if ($is_new) {
				$this->staging_common->clone_table_structure($table, $new_table);
			}
			$this->staging_common->clone_table_content($table, $new_table, $limit, $offset);
			$this->same_server_add_completed_table();
		}
	}

	public function same_server_replace_links() {

		$same_server_replace_old_url = $this->options->get_option('same_server_replace_old_url');
		dark_debug($same_server_replace_old_url , '-------------$same_server_replace_old_url -------------------');

		if(empty($same_server_replace_old_url)){
			$this->replace_old_url();
		}

		$this->create_default_htaccess();

		$new_prefix = $this->get_staging_db_prefix('full_prefix');

		if (empty($new_prefix) || $new_prefix == $this->wpdb->base_prefix) {
			return false;
		}

		$this->disable_search_engine_indexing($new_prefix);

		$this->update_site_and_home_url($new_prefix);

		$this->rewrite_rules($new_prefix);

		$this->update_user_roles($new_prefix);

		//Deactivate WP Time Capsule on staging site
		$this->app_functions->run_deactivate_plugin('wp-time-capsule/wp-time-capsule.php', $new_prefix);

		//Activate WP Time Capsule Staging plugin on staging site
		$this->app_functions->run_activate_plugin('wp-time-capsule-staging/wp-time-capsule-staging.php', $new_prefix);

		//Replace new prefix
		$this->replace_prefix($new_prefix);

		$new_site_url = $this->same_server_staging_url();

		//multisite changes
		if (is_multisite()) {
			$this->staging_common->multi_site_db_changes($new_prefix, $new_site_url);
		}

		//replace $table_prefix in wp-config.php
		$this->staging_common->modify_wp_config($this->same_staging_folder, site_url(), $new_site_url, $new_prefix);

		// Replace path in index.php
		$this->same_server_reset_index_php();

		$this->staging_to_live_compatible();

		dark_debug(array(), '---------COMPLETED------------');
	}

	private function disable_search_engine_indexing($new_prefix){
		//discourage indexing
		$this->create_robot_txt();
		$this->discourage_search_engine($new_prefix, $reset_permalink = true);
		$this->logger->log('Enabled discouraging search engines from indexing', 'staging_progress', $this->staging_id);
	}

	private function update_site_and_home_url($new_prefix){
		$result = $this->wpdb->query(
			$this->wpdb->prepare(
				'UPDATE ' . $new_prefix . 'options SET option_value = %s WHERE option_name = \'siteurl\' OR option_name = \'home\'',
				$this->same_server_staging_url()
			)
		);

		return $result;
	}

	private function rewrite_rules($new_prefix){
		//Update rewrite_rules in clone options table
		$result = $this->wpdb->query(
			$this->wpdb->prepare(
				'UPDATE ' . $new_prefix . 'options SET option_value = %s WHERE option_name = \'rewrite_rules\'',
				''
			)
		);

		if (!$result) {
			dark_debug("Updating option[rewrite_rules] not successfull, likely the main site is not using permalinks", '--------FAILED-------------');
			return ;
		}
	}

	private function update_user_roles($new_prefix){
		$result = $this->wpdb->query(
			"UPDATE  ". $new_prefix . "options SET option_name = '" . $new_prefix . "user_roles' WHERE option_name = '".$this->wpdb->prefix."user_roles' LIMIT 1"
		);

		if (!$result) {
			$error = isset($this->wpdb->error) ? $this->wpdb->error : '';
			dark_debug("User roles modification has been failed", $error , '--------FAILED-------------');
			return ;
		}

		$this->logger->log('User roles has been modified succesfully', 'staging_progress', $this->staging_id);
	}

	//replace table prefix in meta_keys
	private function replace_prefix($new_prefix){
		$usermeta_sql = $this->wpdb->prepare(
				'UPDATE ' . $new_prefix . 'usermeta SET meta_key = REPLACE(meta_key, %s, %s) WHERE meta_key LIKE %s',
				$this->wpdb->base_prefix,
				$new_prefix,
				$this->wpdb->base_prefix . '_%'
			);

		$result_usermeta = $this->wpdb->query( $usermeta_sql );

		$options_sql = $this->wpdb->prepare(
				'UPDATE ' . $new_prefix . 'options SET option_name = REPLACE(option_name, %s, %s) WHERE option_name LIKE %s',
				$this->wpdb->base_prefix,
				$new_prefix,
				$this->wpdb->base_prefix . '_%'
			);

		$result_options = $this->wpdb->query( $options_sql );

		if ($result_options === false || $result_usermeta === false) {
			$this->logger->log("Updating table $new_prefix has been failed.". $this->wpdb->last_error, 'staging_progress', $this->staging_id);
			dark_debug("Updating db prefix $new_prefix has been failed.". $this->wpdb->last_error, '-----------FAILED----------');
			return ;
		}

		$this->logger->log('Updating db prefix "' . $this->wpdb->base_prefix . '" to  "' . $new_prefix . '" has been done succesfully.', 'staging_progress', $this->staging_id);
	}

	private function replace_old_url(){
		$this->same_server_set_staging_path();
		$raw_result = $this->options->get_option('same_server_replace_old_url_data');
		$tables = false;
		if (!empty($raw_result)) {
			$tables = @unserialize($raw_result);
		}

		$old_url = site_url();
		$new_url = $this->same_server_staging_url();
		$old_file_path = WPTC_ABSPATH;
		$new_file_path = wp_normalize_path($this->same_staging_folder);

		dark_debug($old_url, '---------------$old_url-----------------');
		dark_debug($new_url, '---------------$new_url-----------------');
		dark_debug($old_file_path, '---------------$old_file_path-----------------');
		dark_debug($new_file_path, '---------------$new_file_path-----------------');

		$table_prefix = $this->get_staging_db_prefix('full_prefix');

		$this->staging_common->replace_uri($old_url, $new_url, $old_file_path, $new_file_path, $table_prefix, $tables);
	}

	private function create_default_htaccess(){

		$this->logger->log('.htaccess has been modified.', 'staging_progress', $this->staging_id);

		if (is_multisite()) {
			return $this->multi_site_default_htaccess();
		}

		return $this->normal_site_default_htaccess();
	}

	private function multi_site_default_htaccess(){
		$args    = parse_url($this->same_server_staging_url());
		$string  = rtrim($args['path'], "/");
		$data = "\nRewriteBase ".$string."/\nRewriteRule ^index\.php$ - [L]\n\n ## add a trailing slash to /wp-admin\nRewriteRule ^([_0-9a-zA-Z-]+/)?wp-admin$ $1wp-admin/ [R=301,L]\n\nRewriteCond %{REQUEST_FILENAME} -f [OR]\nRewriteCond %{REQUEST_FILENAME} -d\nRewriteRule ^ - [L]\nRewriteRule ^([_0-9a-zA-Z-]+/)?(wp-(content|admin|includes).*) $2 [L]\nRewriteRule ^([_0-9a-zA-Z-]+/)?(.*\.php)$ $2 [L]\nRewriteRule . index.php [L]";
		@file_put_contents($this->same_staging_folder.'.htaccess', $data);
	}

	private function normal_site_default_htaccess(){
		$args    = parse_url($this->same_server_staging_url());
		$string  = rtrim($args['path'], "/");
		$data = "# BEGIN WordPress\n<IfModule mod_rewrite.c>\nRewriteEngine On\nRewriteBase ".$string."/\nRewriteRule ^index\.php$ - [L]\nRewriteCond %{REQUEST_FILENAME} !-f\nRewriteCond %{REQUEST_FILENAME} !-d\nRewriteRule . ".$string."/index.php [L]\n</IfModule>\n# END WordPress";
		@file_put_contents($this->same_staging_folder.'.htaccess', $data);
	}

	private function discourage_search_engine($new_prefix, $reset_permalink = false){
		dark_debug(array(), '--------discourage_search_engine started--------');
		$result = $this->wpdb->query(
			$this->wpdb->prepare(
				'UPDATE ' . $new_prefix . 'options SET option_value = %s WHERE option_name = \'blog_public\'',
				0
			)
		);

		if (!is_multisite()) {
			return false;
		}
		$new_prefix = (string) $new_prefix;
		$wp_tables = $this->processed_files->get_all_tables();
		foreach ($wp_tables as $table) {
			if (stripos($table, 'options') === false || stripos($table, $new_prefix) === false) {
				continue;
			}
			dark_debug($table, '--------$table for turn of indexing--------');
			$this->wpdb->query(
				$this->wpdb->prepare(
					'UPDATE ' . $table . ' SET option_value = %s WHERE option_name = \'blog_public\'',
					0
				)
			);

			if (!$reset_permalink) {
				continue;
			}

			$this->wpdb->query(
				$this->wpdb->prepare(
					'UPDATE ' . $table . ' SET option_value = %s WHERE option_name = \'permalink_structure\'',
					false
				)
			);
		}
	}

	private function create_robot_txt(){
		$data = "User-agent: *\nDisallow: /\n";
		@file_put_contents($this->same_staging_folder.'robots.txt', $data);
	}

	private function same_server_reset_index_php(){
		$path = $this->same_staging_folder . 'index.php';
		$content = file_get_contents($path);

		if ($content) {

			$pattern = "/(require(.*)wp-blog-header.php' \);)/";
			if ( !preg_match($pattern, $content, $matches) ){
				dark_debug(array(), '---------Fatal error: wp-blog-header.php not------------');
			}
			$pattern2 = "/require(.*) dirname(.*) __FILE__ (.*) \. '(.*)wp-blog-header.php'(.*);/";
			$replace = "require( dirname( __FILE__ ) . '/wp-blog-header.php' );// " . $matches[0] . " // Changed by WP Time Capsule";
			$content = preg_replace($pattern2, $replace, $content);

			$content = $this->staging_common->remove_unwanted_comment_lines($content);

			if (FALSE === file_put_contents($path, $content)) {
				dark_debug($path . ' is not writable', '-------FAILED--------------');
			} else {
				dark_debug('Index file updated successfully', '-------FAILED--------------');
			}
		} else {
			dark_debug($path . ' is not writable', '-------FAILED--------------');
		}
	}

	private function same_server_replace_pathname($path){
		return wp_normalize_path(str_replace(WPTC_ABSPATH , $this->same_staging_folder, $path));
	}

	private function same_server_set_staging_path($path = false){
		$path = empty($path) ? $this->options->get_option('same_server_staging_path') : $path;
		$this->same_staging_folder = wp_normalize_path(get_home_path_wptc() . $path . '/');
		$this->same_staging_bridge_dir = wp_normalize_path($this->same_staging_folder . self::CLONE_TMP_FOLDER . '/');
	}

	private function get_staging_db_prefix($type = 'prefix'){
		$prefix = $this->options->get_option('same_server_staging_db_prefix');

		if (empty($prefix)) {
			$prefix = time();
			$full_prefix = $prefix . '_' . $this->wpdb->base_prefix;
			$this->options->set_option('same_server_staging_db_prefix', $prefix);
			$this->options->set_option('same_server_staging_full_db_prefix', $full_prefix);
		}

		if ($type === 'prefix') {
			return $prefix;
		}

		return $this->options->get_option('same_server_staging_full_db_prefix');
	}

	public function get_existing_staging_prefix(){
		$prefix = $this->get_staging_details('db_prefix');
		$full_prefix = $prefix . $this->wpdb->base_prefix;
		if (empty($prefix) || $prefix == $this->wpdb->base_prefix) {
			return false;
		}
		return $full_prefix;
	}

	private function same_server_staging_url(){
		return get_home_url() . '/' . $this->options->get_option('same_server_staging_path');
	}

	public function same_server_staging_bridge_url(){
		return get_home_url() . '/' . $this->options->get_option('same_server_staging_path') . '/' . self::CLONE_TMP_FOLDER . '/' . 'bridge.php' ;
	}

	public function save_staging_settings($data){

		if (!empty($data['db_rows_clone_limit_wptc'])) {
			$this->config->set_option('internal_staging_db_rows_copy_limit', $data['db_rows_clone_limit_wptc']);
		}

		if (!empty($data['files_clone_limit_wptc'])) {
			$this->config->set_option('internal_staging_file_copy_limit', $data['files_clone_limit_wptc']);
		}

		if (!empty($data['deep_link_replace_limit_wptc'])) {
			$this->config->set_option('internal_staging_deep_link_limit', $data['deep_link_replace_limit_wptc']);
		}

		if (!empty($data['disable_admin_login_wptc'])) {
			$this->config->set_option('internal_staging_disable_admin_login', $data['disable_admin_login_wptc']);
		} else {
			$this->config->set_option('internal_staging_disable_admin_login', false);
		}
	}

	private function staging_to_live_compatible(){
		$this->staging_to_live_copy_files();
		$this->staging_to_live_set_flags();
		$this->staging_to_live_copy_db();
	}

	public function staging_to_live_copy_files(){

		$this->init_fs();
		$this->same_server_set_staging_path();

		$stage_wp_content = $this->same_server_replace_pathname(WPTC_WP_CONTENT_DIR);

		if (WPTC_WP_CONTENT_DIR === $stage_wp_content) {
			dark_debug(array(), '--------Dont copy to live site--------');
			return false;
		}

		$stage_plugins_dir = $stage_wp_content . '/plugins/' . WPTC_STAGING_PLUGIN_DIR_NAME;

		//Key is Live path and Value is Staging path
		$files = array(
			WPTC_PLUGIN_DIR 	. 'Pro/Staging/class-stage-common.php' => $stage_plugins_dir . '/includes/class-stage-common.php',
			WPTC_PLUGIN_DIR 	. 'Pro/Staging/stage-to-live/wp-time-capsule-staging.php' => $stage_plugins_dir . '/wp-time-capsule-staging.php',
			WPTC_PLUGIN_DIR 	. 'common-functions.php' 	=> $stage_plugins_dir . '/includes/common-functions.php' ,
			WPTC_PLUGIN_DIR 	. 'wptc-constants.php' 		=> $stage_plugins_dir . '/wptc-constants.php' ,
			WPTC_CLASSES_DIR 	. 'class-file-iterator.php' => $stage_plugins_dir . '/includes/class-file-iterator.php',
			WPTC_PLUGIN_DIR 	. 'tc-ui.css'				=> $stage_plugins_dir . '/tc-ui.css' ,
			WPTC_PLUGIN_DIR 	. 'wp-time-capsule.css' 	=> $stage_plugins_dir . '/wp-time-capsule.css' ,
			);

		if (file_exists(WPTC_PLUGIN_DIR . 'wptc-env-parameters.php')) {
			$files[WPTC_PLUGIN_DIR . 'wptc-env-parameters.php'] = $stage_plugins_dir . '/wptc-env-parameters.php';
		}

		//Key is Live path and Value is Staging path
		$dirs = array(
			WPTC_PLUGIN_DIR . 'Classes' => $stage_plugins_dir . '/Classes',
			WPTC_PLUGIN_DIR . 'Base' 	=> $stage_plugins_dir .'/Base',
			WPTC_PLUGIN_DIR . 'fonts' 	=> $stage_plugins_dir .'/fonts',
			WPTC_PLUGIN_DIR . 'utils' 	=> $stage_plugins_dir . '/utils',
			WPTC_PLUGIN_DIR . 'Pro/Staging/stage-to-live/' => $stage_plugins_dir . '/',
			);

		foreach ($dirs as $live_path => $stage_path) {
			$this->file_iterator->copy_dir($live_path, $stage_path);
		}

		foreach ($files as $live_path => $stage_path) {
				$this->filesystem->copy($live_path, $stage_path, true, FS_CHMOD_FILE);
		}
	}

	public function staging_to_live_set_flags(){
		$this->config->set_option('s2l_live_path', WPTC_ABSPATH);
		$this->config->set_option('s2l_live_url', site_url());
		$this->config->set_option('s2l_live_db_prefix', $this->wpdb->base_prefix);
		$this->config->set_option('s2l_live_permalink_structure', get_option('permalink_structure'));
		$this->subdir_installation_related();
	}

	private function subdir_installation_related(){
		if(get_home_path_wptc() != WPTC_ABSPATH){
			$this->config->set_option('s2l_is_subdir_installation', true);
		}
	}

	public function staging_to_live_copy_db(){
		$tables = array(
			$this->wpdb->base_prefix . 'wptc_processed_iterator' => 'schema',
			$this->wpdb->base_prefix . 'wptc_current_process' => 'schema',
			$this->wpdb->base_prefix . 'wptc_options' => 'full',
			);

		foreach ($tables as $table => $type) {
			$db_prefix = $this->get_staging_db_prefix('prefix');
			$new_table  = $db_prefix . '_' . $table;
			$this->staging_common->clone_table_structure($table, $new_table);

			if (empty($type) || $type !== 'full' || $type === 'schema') {
				continue;
			}

			$this->staging_common->clone_table_content($table, $new_table, $limit = WPTC_STAGING_DEFAULT_COPY_DB_ROWS_LIMIT, $offset = 0);
		}
	}
}
