<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class WPTC_Stage_To_Live{
	private $file_iterator;
	private $config;
	private $current_iterator_table;
	private $filesystem;
	private $stage_path;
	private $live_path;
	private $app_functions;
	private $copy_file_offset;
	private $file_base;
	private $processed_files;
	private $wpdb;
	private $staging_common;
	private $staging_id;

	public function __construct(){
		$this->file_iterator = new WPTC_File_Iterator();
		$this->config = WPTC_Factory::get('config');
		$this->current_iterator_table = new WPTC_Processed_iterator();
		$this->stage_path = WPTC_ABSPATH;
		$this->app_functions = WPTC_Base_Factory::get('Wptc_App_Functions');
		$this->file_base = new Utils_Base();
		$this->processed_files = WPTC_Factory::get('processed-files');
		$this->staging_common = new WPTC_Stage_Common();
		$this->init_fs();
		$this->init_db();
		$this->set_live_path();
	}

	public function to_live(){
		$this->app_functions->set_start_time();
		$this->choose_action();
	}

	public function init_db(){
		$this->wpdb = $this->staging_common->init_db();
	}

	private function init_fs(){
		$this->filesystem = $this->staging_common->init_fs();
	}

	private function init_staging_id(){
		$this->staging_id = $this->staging_common->init_staging_id();
	}

	private function choose_action(){

		$this->init_staging_id();

		$current_action = $this->config->get_option('s2l_current_action', true);
		$current_action = empty($current_action)? 'first_action' : $current_action;

		dark_debug($current_action, '--------$current_action--------');

		switch ($current_action) {
			case 'first_action':
				$this->flush_all_flags();
				$this->setup();
				$this->clone_db();
				break;
			case 'replace_links':
				$this->replace_links();
				break;
			case 'get_folders':
				$this->get_folders();
				break;
			case 'get_hash_by_folders':
				$this->get_hash_by_folders();
				break;
			case 'copy_files_to_live':
				$this->copy_files_to_live();
				break;
			case 'final_things_to_do';
				$this->final_things_to_do();
				break;
			default:
				dark_debug(array(), '--------Something went wrong--------');
				break;
		}

	}

	private function clone_db(){
		$tables = $this->get_all_tables();
		$limit = 1000;
		dark_debug($tables, '--------$tables--------');
		$live_prefix =  $this->config->get_option('s2l_live_db_prefix');

		foreach ($tables as $table) {
			dark_debug($table, '--------$table--------');
			$new_table = str_replace($this->wpdb->base_prefix, $live_prefix, $table);
			dark_debug($new_table, '--------$new_table--------');

			if ($this->current_iterator_table->is_complete($table)) {
				continue;
			}

			$table_meta = $this->staging_common->get_table_data($table);
			extract($table_meta);

			if ($is_new) {
				$this->staging_common->clone_table_structure($table, $new_table);
			}

			$this->staging_common->clone_table_content($table, $new_table, $limit, $offset);

			$this->current_iterator_table->update_iterator($table, -1); //Done
		}

		$this->config->set_option('s2l_current_action', 'replace_links');
		$this->app_functions->die_with_json_encode(array('status' => 'continue', 'msg' => 'Database is cloned', 'percentage' => 10));
	}

	private function get_all_tables(){
		$wp_tables = $this->processed_files->get_all_tables();
		$site_tables = array();
		foreach ($wp_tables as $table) {
			if(strpos($table, $this->wpdb->base_prefix) !== false){
				$site_tables[] = $table;
			}
		}

		return $site_tables;
	}

	private function replace_links(){
		$this->replace_deep_links();

		$this->replace_site_links();

		$this->config->set_option('s2l_current_action', 'get_folders');
		$this->app_functions->die_with_json_encode(array('status' => 'continue', 'msg' => 'Database links are replaced.', 'percentage' => 20));
	}

	private function replace_site_links(){
		$new_prefix =  $this->config->get_option('s2l_live_db_prefix');
		$new_url = $this->config->get_option('s2l_live_url');

		$result = $this->wpdb->query(
			$this->wpdb->prepare(
				'update ' . $new_prefix . 'options set option_value = %s where option_name = \'siteurl\' or option_name = \'home\'',
				$new_url
			)
		);

		if (!$result) {
			$error = isset($this->wpdb->error) ? $this->wpdb->error : '';
			// $this->logger->log('Replacing site url has been failed.' . $error, 'staging_progress', $this->staging_id);
			dark_debug('Replacing site url has been failed. ' . $error, '--------FAILED----------');
		} else {
			// $this->logger->log('Replacing siteurl has been done succesfully', 'staging_progress', $this->staging_id);
			dark_debug('Replacing siteurl has been done succesfully', '--------SUCCESS----------');
		}

		//Update rewrite_rules in clone options table
		$result = $this->wpdb->query(
			$this->wpdb->prepare(
				'update ' . $new_prefix . 'options set option_value = %s where option_name = \'rewrite_rules\'',
				''
			)
		);

		if (!$result) {
			dark_debug("Updating option[rewrite_rules] not successfull, likely the main site is not using permalinks", '--------FAILED-------------');
		} else {
			dark_debug("Updating option [rewrite_rules] has been done succesfully", '--------SUCCESS-------------');
		}

		$result = $this->wpdb->query(
			$this->wpdb->prepare(
				"update  ". $new_prefix . "options set option_name = '" . $new_prefix . "user_roles' where option_name = '".$this->wpdb->prefix."user_roles' limit 1", ''
			)
		);

		if (!$result) {
			$error = isset($this->wpdb->error) ? $this->wpdb->error : '';
			// $this->logger->log('User roles modification has been failed.' . $error, 'staging_progress', $this->staging_id);
			dark_debug("User roles modification has been failed", '--------FAILED-------------');
		} else {
			// $this->logger->log('User roles has been modified succesfully', 'staging_progress', $this->staging_id);
			dark_debug("User roles has been modified succesfully", '--------SUCCESS-------------');
		}


		//replace table prefix in meta_keys
		$usermeta_sql = $this->wpdb->prepare(
				'update ' . $new_prefix . 'usermeta set meta_key = replace(meta_key, %s, %s) where meta_key like %s',
				$this->wpdb->base_prefix,
				$new_prefix,
				$this->wpdb->base_prefix . '_%'
			);
		dark_debug($usermeta_sql, '--------$usermeta_sql--------');

		$result_usermeta = $this->wpdb->query( $usermeta_sql );

		dark_debug($result_usermeta, '--------$result_usermeta--------');

		$options_sql = $this->wpdb->prepare(
				'update ' . $new_prefix . 'options set option_name = replace(option_name, %s, %s) where option_name like %s',
				$this->wpdb->base_prefix,
				$new_prefix,
				$this->wpdb->base_prefix . '_%'
			);
		$result_options = $this->wpdb->query( $options_sql );
		dark_debug($options_sql, '--------$options_sql--------');
		dark_debug($result_options, '--------$result_options--------');

		if ($result_options === false || $result_usermeta === false) {
			// $this->logger->log("Updating table $new_prefix has been failed.". $this->wpdb->last_error, 'staging_progress', $this->staging_id);
			dark_debug("Updating db prefix $new_prefix has been failed.". $this->wpdb->last_error, '-----------FAILED----------');
		} else {
			// $this->logger->log('Updating db prefix "' . $this->wpdb->base_prefix . '" to  "' . $new_prefix . '" has been done succesfully.', 'staging_progress', $this->staging_id);
			dark_debug('Updating db prefix "' . $this->wpdb->base_prefix . '" to  "' . $new_prefix . '" has been done succesfully.', '--SUCCESS-------------------');
		}

	}

	private function replace_deep_links(){
		$replace_deep_links = $this->config->get_option('s2l_deep_links_completed');

		if ($replace_deep_links) {
			return false;
		}

		$raw_result = $this->config->get_option('same_server_replace_old_url_data');
		$tables = false;
		if (!empty($raw_result)) {
			$tables = @unserialize($raw_result);
		}

		$old_url = site_url();
		$new_url = $this->config->get_option('s2l_live_url');
		$old_file_path = $this->stage_path;
		$new_file_path = $this->live_path;

		dark_debug($old_url, '---------------$old_url-----------------');
		dark_debug($new_url, '---------------$new_url-----------------');
		dark_debug($old_file_path, '---------------$old_file_path-----------------');
		dark_debug($new_file_path, '---------------$new_file_path-----------------');
		$table_prefix =  $this->config->get_option('s2l_live_db_prefix');

		$this->staging_common->replace_uri($old_url, $new_url, $old_file_path, $new_file_path, $table_prefix, $tables);

		$this->config->set_option('s2l_deep_links_completed', true);

	}

	private function get_folders(){
		$this->file_iterator->get_folders();
		$this->config->set_option('s2l_current_action', 'get_hash_by_folders');
		$this->app_functions->die_with_json_encode(array('status' => 'continue', 'msg' => 'Processing files...', 'percentage' => 25));
	}

	private function get_hash_by_folders(){

		$break = false;
		$loop = $temp_counter = 0;

		while(!$break){
			$dir_meta = $this->current_iterator_table->get_unfnished_folder();
			// dark_debug($dir_meta, '--------$dir_meta--------');
			$deep_dirs = false;
			if (empty($dir_meta) || $dir_meta->count === -1) {
				$break = true;
				continue;
			}

			$path = wp_normalize_path($dir_meta->name);

			if( array_search($path, $this->file_iterator->get_deep_dirs()) !== false ){
				$deep_dirs = true;
			}

			// dark_debug(array(), '--------Running--------');
			if(wptc_is_dir($path)){
				$this->get_hash_dir($path, $dir_meta->count, $temp_counter, $deep_dirs);
			} else {
				$this->get_hash_file($path, $dir_meta->count, $temp_counter);
			}
		}
		$this->config->set_option('s2l_current_action', 'copy_files_to_live');
		$this->app_functions->die_with_json_encode(array('status' => 'continue', 'msg' => 'Coping files...', 'percentage' => 50));
	}

	private function get_hash_dir($path, $offset, &$temp_counter, $deep_dirs){

		$counter = 0;
		$timeout_limit = 500;
		$qry = '';

		if ($deep_dirs) {
			$files_obj = $this->file_iterator->get_files_obj_by_path($path, $recursive = false);
		} else {
			$files_obj = $this->file_iterator->get_files_obj_by_path($path, $recursive = true);
		}

		foreach ($files_obj as $file_obj) {
			if ($counter++ < $offset ) {
				continue;
			}

			$offset++;
			$file = $file_obj->getPathname();
			$file = wp_normalize_path($file);

			if ($deep_dirs && wptc_is_dir($file)){
				dark_debug($file, '--------skip because of deep dir--------');
				continue;
			}

			// dark_debug($file, '--------$file_obj->getPathname()--------');

			if ($this->is_skip($file)) {
				// dark_debug(array(), '--------SKIP FILE 1--------');
				$this->app_functions->check_timeout_iter_file($path, $temp_counter, $timeout_limit, $qry, $offset);
				continue;
			}


			if(!$this->is_file_modified($file)){
				// dark_debug(array(), '--------SKIP FILE 2--------');
				$this->app_functions->check_timeout_iter_file($path, $temp_counter, $timeout_limit, $qry, $offset);
				continue;
			}

			dark_debug(array(), '--------File Modified--------');

			if (empty($qry)) {
				$qry .= "('";
			} else {
				$qry .= ",('";
			}

			$qry .= wp_normalize_path($file) . "', 'Q', NULL)";

			$this->app_functions->check_timeout_iter_file($path, $temp_counter, $timeout_limit, $qry, $offset);
		}

		if (!empty($qry)) {
			$this->app_functions->insert_into_current_process($qry);
		}

		$this->current_iterator_table->update_iterator($path, -1);
	}

	private function get_hash_file($file){
		$file = wp_normalize_path($file);

		if ($this->is_skip($file)) {
			// dark_debug(array(), '--------SKIP FILE 1--------');
			$this->current_iterator_table->update_iterator($file, -1);
			return ;
		}


		if(!$this->is_file_modified($file)){
			// dark_debug(array(), '--------SKIP FILE 2--------');
			$this->current_iterator_table->update_iterator($file, -1);
			return ;
		}

		$qry = "('";
		$qry .= wp_normalize_path($file) . "', 'Q', NULL)";


		if (!empty($qry)) {
			$this->app_functions->insert_into_current_process($qry);
		}

		$this->current_iterator_table->update_iterator($file, -1);
	}

	private function is_file_modified($stage_file){
		if(empty($stage_file))
			return false;

		$live_file = str_replace($this->stage_path, $this->live_path, $stage_file);

		return $this->is_hash_modified($stage_file, $live_file);
	}

	private function is_hash_modified($stage_file, $live_file){
		if (!file_exists($live_file)) {
			return true;
		}

		if(!wptc_is_hash_required($stage_file)){
			dark_debug(array(), '---------cannot find has so checking is_size_mismatch------------');
			return $this->is_size_mismatch($stage_file, $live_file);
		}

		$stage_file_hash = wptc_get_hash($stage_file);
		$live_file_hash  = wptc_get_hash($live_file);

		if ($stage_file_hash == $live_file_hash) {
			return false;
		}

		return true;
	}

	private function is_skip($file){
		if(wptc_is_dir($file)){
			return true;
		}

		if($this->is_wptc_dir($file)){
			return true;
		}

		if (strpos($file, '.htaccess') !== false) {
			return true;
		}

		return false;

	}

	private function is_wptc_dir($file){
		if(strpos($file, WPTC_PLUGIN_DIR) !== false){
			return true;
		}

		return false;
	}

	private function is_size_mismatch($stage_file, $live_file){

		if(!is_readable($stage_file) || !is_readable($live_file))
			return false;

		if(filesize($stage_file) == filesize($live_file))
			return false;

		return true;
	}

	private function set_live_path(){
		if (!empty($this->live_path)) {
			return false;
		}
		$this->live_path = $this->config->get_option('s2l_live_path');
		dark_debug($this->live_path, '--------$this->live_path--------');
	}

	private function get_queued_files(){

		$offset = $this->config->get_option('s2l_copy_files_offset');
		dark_debug(array(), '--------$offset--------');
		$offset = empty($offset) ? 0 : $offset;
		$this->copy_file_offset = $offset;

		$sql = "SELECT file_path FROM `".$this->wpdb->base_prefix."wptc_current_process` WHERE `status` = 'Q' ORDER BY `id` LIMIT " . $offset . ", " . WPTC_STAGING_DEFAULT_FILE_COPY_LIMIT . "";
		// dark_debug($sql, '---------$sql------------');

		return $this->wpdb->get_results($sql, ARRAY_N);
		// dark_debug($result, "--------result--------");
	}

	private function copy_files_to_live(){

		$queue_files = $this->get_queued_files();

		if(empty($queue_files)){
			$this->config->set_option('s2l_current_action', 'final_things_to_do');
			$this->app_functions->die_with_json_encode(array('status' => 'continue', 'msg' => 'Files are copied successfully.', 'percentage' => 90));
		}

		dark_debug($queue_files, '--------$queue_files--------');

		$counter = 0;
		foreach ($queue_files as $key => $file) {
			$counter++ ;
			$stage_file = $file[0];

			dark_debug($this->stage_path, '--------$this->stage_path--------');
			dark_debug($this->live_path, '--------$this->live_path--------');

			$live_file = str_replace($this->stage_path, $this->live_path, $stage_file);

			dark_debug($stage_file, '--------$file--------');
			dark_debug($live_file, '--------$live_file--------');

			$live_dir = dirname($live_file);

			$this->make_dir_by_path($live_dir);

			if (!file_exists($live_dir)) {
				dark_debug($live_dir, '--------Could not create folder - --------');
					continue;
			}

			if(filesize($stage_file) < WPTC_STAGING_COPY_SIZE){

				$copy_status = $this->filesystem->copy($file[0], $live_file, true, FS_CHMOD_FILE);
				dark_debug($copy_status, '--------$copy_status--------');
				if (!$copy_status)
					dark_debug(array(), '--------Could not copy this file--------');

			} else {
				$copy_status = wptc_copy_large_file($stage_file, $live_file);
				dark_debug($copy_status, '--------$copy_status--------');
				if (!$copy_status)
					dark_debug(array(), '--------Could not copy this file--------');
			}

			$break = is_wptc_timeout_cut();
			if ($break) {
				$this->config->set_option('s2l_copy_files_offset', $this->copy_file_offset + $counter);
				$this->app_functions->die_with_json_encode(array("status" => "continue", 'msg' => 'Copying files ('. ($this->copy_file_offset + $counter) .')', 'percentage' => 80));
			}
		}
		dark_debug($counter, '--------$counter--------');
		dark_debug($this->copy_file_offset, '--------$this->copy_file_offset--------');
		$this->config->set_option('s2l_copy_files_offset', $this->copy_file_offset + $counter);
		$this->app_functions->die_with_json_encode(array("status" => "continue", 'msg' => 'Copying files ('. ($this->copy_file_offset + $counter) .')', 'percentage' => 80));
	}

	private function make_dir_by_path($path, $recursive = true){
		$path = wp_normalize_path($path);
		$this->file_base->createRecursiveFileSystemFolder($path, false, false);
	}

	private function final_things_to_do(){
		$new_prefix = $this->config->get_option('s2l_live_db_prefix');
		$new_site_url = $this->config->get_option('s2l_live_url');

		//change wp-config.php
		$this->change_wp_config($new_prefix, $new_site_url);
		dark_debug(array(), '--------1--------');
		$this->remove_robot_txt();

		$this->encourage_search_engine($new_prefix);

		//multisite changes
		if (is_multisite()) {
			$this->staging_common->multi_site_db_changes($new_prefix, $new_site_url);
		}

		$this->completed();
		$this->app_functions->die_with_json_encode(array("status" => "completed", 'msg' => 'Copied to live site successfully !', 'percentage' => 100, 'time' => $this->get_last_time_copy_to_live()));
	}

	private function encourage_search_engine($new_prefix){

		if (!is_multisite()) {
			$table = $new_prefix . 'options';
			$this->revert_search_engine($table);
			$this->revert_permalink($table);
			return false;
		}

		$new_prefix = (string) $new_prefix;
		$wp_tables = $this->processed_files->get_all_tables();

		foreach ($wp_tables as $table) {
			if (stripos($table, 'options') === false || stripos($table, $new_prefix) !== 0) {
				continue;
			}

			$this->revert_search_engine($table);
			$this->revert_permalink($table);
		}
	}

	private function revert_permalink($table){
		$old_permalink_structure = $this->config->get_option('s2l_live_permalink_structure');
		$result = $this->wpdb->query(
			$this->wpdb->prepare(
				'update ' . $table . ' set option_value = %s where option_name = \'permalink_structure\'',
				$old_permalink_structure
			)
		);
	}

	private function revert_search_engine($table){
		$result = $this->wpdb->query(
			$this->wpdb->prepare(
				'update ' . $table . ' set option_value = %s where option_name = \'blog_public\'',
				1
			)
		);
	}

	private function change_wp_config($new_prefix, $new_site_url){
		$this->staging_common->modify_wp_config($this->live_path, site_url(), $new_site_url, $new_prefix);
	}

	private function remove_robot_txt(){
		$delete_status = $this->filesystem->delete($this->live_path.'robots.txt');
		dark_debug($delete_status, '--------$delete_status--------');
	}

	private function completed(){
		$live_prefix =  $this->config->get_option('s2l_live_db_prefix');

		//deactivate staging plugin in live site
		$this->app_functions->run_deactivate_plugin('wp-time-capsule-staging/wp-time-capsule-staging.php', $live_prefix);
		$this->app_functions->run_activate_plugin('wp-time-capsule/wp-time-capsule.php', $live_prefix);

		WPTC_Base_Factory::get('Wptc_Backup_Analytics')->send_report_data($this->staging_id, 'STAGING_TO_LIVE', 'SUCCESS');

		$this->config->set_option('s2l_last_copied_time', time());
		$this->config->set_option('s2l_current_action', false);
		$this->config->set_option('same_server_replace_old_url', false);
		$this->config->set_option('same_server_replace_old_url_data', false);
		$this->config->set_option('same_server_replace_url_multicall_status', false);
		$this->config->set_option('s2l_copy_files_offset', false);
		$this->config->set_option('s2l_deep_links_completed', false);
		$this->config->set_option('same_server_clear_flags', false);
		$this->config->set_option('same_server_setup', false);
		$this->wpdb->query("TRUNCATE TABLE `".$this->wpdb->base_prefix."wptc_current_process`;");
		$this->wpdb->query("TRUNCATE TABLE `".$this->wpdb->base_prefix."wptc_processed_iterator`;");
	}

	private function flush_all_flags(){
		if ($this->config->get_option('same_server_clear_flags')) {
			return false;
		}

		$this->config->set_option('same_server_replace_old_url', false);
		$this->config->set_option('same_server_replace_old_url_data', false);
		$this->config->set_option('same_server_replace_url_multicall_status', false);
		$this->config->set_option('same_server_clear_flags', true);
	}

	private function setup(){
		if ($this->config->get_option('same_server_setup')) {
			return false;
		}

		dark_debug(array(), '--------SETUP STARTED--------');

		WPTC_Base_Factory::get('Wptc_Backup_Analytics')->send_report_data($this->staging_id, 'STAGING_TO_LIVE', 'STARTED');

		$this->config->set_option('same_server_setup', true);
	}

	public function check_permissions() {
		if (!$this->should_disable_login())
			return false;

		wp_die( sprintf ( __('Access denied. You need to <a href="%1$s" target="_blank">Login</a> first','wptc'), wp_login_url()  ) );
	}

	private function should_disable_login(){
		return (
			$this->is_staging_site() &&
			$this->staging_completed() &&
			$this->config->get_option('internal_staging_disable_admin_login') &&
			(!current_user_can("administrator") && !$this->is_login_page() && !is_admin())
		);
	}

	private function is_staging_site(){
		return true;
	}

	private function staging_completed(){
		$live_prefix = $this->config->get_option('s2l_live_db_prefix');

		$sql = "SELECT value FROM `".$live_prefix."wptc_options` WHERE `name` = 'same_server_staging_status'";
		dark_debug($sql, '--------$sql--------');
		$status = $this->wpdb->get_var($sql);
		dark_debug($status, '--------$status--------');

		if ($status === 'staging_completed') {
			return true;
		}
	}

	private function is_login_page(){
		return (in_array($GLOBALS["pagenow"], array("wp-login.php")));
	}

	public function get_last_time_copy_to_live(){
		$time = $this->config->get_option('s2l_last_copied_time');
		if(empty($time))
			return "none";

		return user_formatted_time_wptc($time);
	}

	public function change_sitename() {

		if(!$this->is_staging_site() ) 	return false;

		global $wp_admin_bar;
		// Main Title
		$wp_admin_bar->add_menu( array(
			'id' => 'site-name',
			'title' => is_admin() ? ('STAGING - ' . get_bloginfo( 'name' ) ) : ( 'STAGING - ' . get_bloginfo( 'name' ) . ' Dashboard' ),
			'href' => is_admin() ? home_url( '/' ) : admin_url(),
		) );
	}
}