<?php

class WPTC_Restore_App_Functions {

	private $config;
	const SECRET_HEAD = '<wptc_head>';
	const SECRET_TAIL = '</wptc_head>';

	public function __construct(){
	}

	public function enable_php_errors(){
		error_reporting(E_ERROR | E_PARSE);
		ini_set('display_errors', 'On');
	}

	public function init_other_functions(){
		wptc_set_time_limit(0);
	}

	public function start_request_time(){
		global $start_time_tc;
		$start_time_tc = microtime(true);
	}

	public function define_constants($enable_bridge_alone = true){
		//used in wptc-constants.php
		define('WPTC_BRIDGE', true);

		if ($enable_bridge_alone) {
			return;
		}

		if (!defined('FS_CHMOD_FILE')) {
			define('FS_CHMOD_FILE', 0644);
		}

		if (!defined('FS_CHMOD_DIR')) {
			define('FS_CHMOD_DIR', 0755);
		}
	}

	public function init_db_connection(){
		//initialize wpdb since we are using it independently
		global $wpdb;
		$wpdb = new wpdb(DB_USER, DB_PASSWORD, DB_NAME, DB_HOST);

		//setting the prefix from post value;
		$wpdb->prefix = $wpdb->base_prefix = DB_PREFIX_WPTC;
	}

	public function init_other_objects(){
		if (!empty($this->config)) {
			return ;
		}

		$this->config = WPTC_Factory::get('config');
	}

	public function init_file_system(){
		$credentials = request_filesystem_credentials("", "", false, false, null);
		if (false === $credentials) {
			$this->die_with_msg(array('error' => 'Filesystem error: Could not get filesystem credentials.'));
		}

		if (!WP_Filesystem($credentials)) {
			$this->die_with_msg(array('error' => 'Filesystem error: Could not initiate filesystem.'));
		}

		return $credentials;
	}

	public function decode_request(){
		// dark_debug($_REQUEST, '--------$_REQUEST--------');
		// if (empty($_REQUEST) || empty($_REQUEST['data'])) {
		// 	return false;
		// }
		// $decode_data = base64_decode($_REQUEST['data']);
		// if (empty($decode_data)) {
		// 	return false;
		// }
		// $unserialize = unserialize($decode_data);
		// if (empty($unserialize)) {
		// 	return false;
		// }
		// dark_debug($unserialize, '--------$unserialize--------');
		// return $unserialize;
		return $_POST;
	}

	public function die_with_msg($msg){
		$json_encoded_msg = json_encode($msg);
		$msg_with_secret = self::SECRET_HEAD . $json_encoded_msg . self::SECRET_TAIL;
		die($msg_with_secret);
	}

	public function init_log_files(){
		$tcapsule_path = $this->config->get_option('backup_db_path');
		$tcapsule_restore_path = $tcapsule_path . '/wptc_restore_logs/';
		$this->create_folder_by_path($tcapsule_restore_path);
		$this->create_golden_file($tcapsule_restore_path);
		$this->create_log_file($tcapsule_restore_path, $type = 'queries');
		$this->create_log_file($tcapsule_restore_path, $type = 'files');
	}

	public function create_folder_by_path($path){
		$base = new Utils_Base();
		$base->createRecursiveFileSystemFolder($path);
	}

	public function create_golden_file($dir_path){
		global $wp_filesystem;
		$wp_filesystem->put_contents($dir_path. 'index.php', '<?php //silence is golden'); // to create a file
	}

	public function create_log_file($dir_path, $type){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		if (empty($dir_path) || empty($type)) {
			return false; //we cannot handle if anyone is false
		}

		global $wp_filesystem;

		if ($type === 'queries') {
			$is_restore_failed_queries_file_created = $this->config->get_option('is_restore_failed_queries_file_created');
			$file_path = $restore_failed_queries_file_path = $this->config->get_option('restore_failed_queries_file_path');

			if(!$is_restore_failed_queries_file_created || !$restore_failed_queries_file_path || !$wp_filesystem->exists($restore_failed_queries_file_path)) {
				$file_name = WPTC_Factory::secret('failed_queries');
				$file_path = $dir_path . $file_name . '.sql';
				$wp_filesystem->put_contents($file_path, ''); // to create a file
				$this->config->set_option('is_restore_failed_queries_file_created', true);
				$this->config->set_option('restore_failed_queries_file_path', $file_path);
			}

		} else if($type === 'files'){
			$is_restore_failed_downloads_file_created = $this->config->get_option('is_restore_failed_downloads_file_created');
			$file_path = $restore_failed_downloads_file_path = $this->config->get_option('restore_failed_downloads_file_path');

			if(!$is_restore_failed_downloads_file_created || !$restore_failed_downloads_file_path || ($restore_failed_downloads_file_path && !$wp_filesystem->exists($restore_failed_downloads_file_path)) ) {
				$file_name = WPTC_Factory::secret('failed_downloads');
				$file_path = $dir_path . $file_name . '.txt';
				$wp_filesystem->put_contents($file_path, ''); // to create a file
				$this->config->set_option('is_restore_failed_downloads_file_created', true);
				$this->config->set_option('restore_failed_downloads_file_path', $file_path);
			}
		}

		return $file_path;
	}

	public function log_data($type, $data){
		if (empty($type) || empty($data)) {
			return false;
		}

		if ($type === 'files') {
			$file_path = $this->config->get_option('restore_failed_downloads_file_path');
		} else if($type === 'queries') {
			$file_path = $this->config->get_option('restore_failed_queries_file_path');
		}

		if (empty($file_path) || !file_exists($file_path)) {
			dark_debug($file_path, '--------$file_path not exist so cannot log--------');
			return false;
		}

		if ($type === 'files') {
			foreach ($data as $key => $value) {
				file_put_contents($file_path, $key . " : " . $value . "\n", FILE_APPEND);
			}
			file_put_contents($file_path, "\n", FILE_APPEND);
		} else if($type === 'queries') {
			file_put_contents($file_path, $data . "\n", FILE_APPEND);
		}

	}

	public function get_failure_data(){
		$restore_failed_queries_file_path =  $this->config->get_option('restore_failed_queries_file_path');
		$restore_failed_downloads_file_path = $this->config->get_option('restore_failed_downloads_file_path');

		$result = array();

		if (!empty($restore_failed_downloads_file_path) && file_exists($restore_failed_downloads_file_path) && filesize($restore_failed_downloads_file_path) > 0) {
			$result['failed_files'] = str_replace(ABSPATH, $this->config->get_option('site_url_wptc'). '/', $restore_failed_downloads_file_path);
		}

		if (!empty($restore_failed_queries_file_path) && file_exists($restore_failed_queries_file_path) && filesize($restore_failed_queries_file_path) > 0) {
			$result['failed_queries'] = str_replace(ABSPATH, $this->config->get_option('site_url_wptc'). '/', $restore_failed_queries_file_path);
		}

		return $result;
	}

	public function check_and_record_not_safe_for_write($this_file) {
		global $wp_filesystem;

		$this_file = $this->config->wp_filesystem_safe_abspath_replace($this_file);

		if($wp_filesystem->is_dir($this_file)){
			return true;
		}
		$this_file = rtrim($this_file, '/');

		if (!$wp_filesystem->exists($this_file) ){
			return true;
		}

		if($wp_filesystem->is_writable($this_file)){
			return true;
		}

		$chmod_result = $wp_filesystem->chmod($this_file, 0644);
		if (!$chmod_result || !$wp_filesystem->is_writable($this_file)) {
			$this->config->save_encoded_not_safe_for_write_files($this_file);
			return false;
		}

		return true;
	}

	public function maybe_call_again_tc($return = false) {
		global $start_time_tc;

		if (!defined('WPTC_TIMEOUT')) {
			define('WPTC_TIMEOUT', 21);
		}

		if ((microtime(true) - $start_time_tc) >= WPTC_TIMEOUT) {

			if ($return) return true;

			$this->die_with_msg("wptcs_callagain_wptce");

		}

		return false;
	}

	public function is_file_hash_same($file_path, $prev_file_hash ,$prev_file_size, $prev_file_mtime = 0) {

		$this->init_other_objects();

		// if($this->config->get_option('is_staging_running') && stripos($file_path, '.htaccess') !== false ){
		// 	dark_debug(array(), '-----htaccess file so not downloaded in staging----------------');
		// 	return true; //do not restore .htaccess if its staging
		// }
		$file_path = $this->config->wp_filesystem_safe_abspath_replace($file_path);
		$file_path = rtrim($file_path, '/');

		if (!file_exists($file_path)) {
			return false;
		}

		if(!wptc_is_hash_required($file_path)){
			dark_debug(array(), '---------cannot find has so checking file size and m time------------');
			return $this->is_same_size_and_same_mtime($file_path, $prev_file_size, $prev_file_mtime);
		}

		$new_file_hash = wptc_get_hash($file_path);
		dark_debug($new_file_hash, '---------$new_file_hash------------');
		if ($prev_file_hash != $new_file_hash) {
			return false;
		}

		return true;

	}

	private function is_same_size_and_same_mtime($file_path, $prev_file_size, $prev_file_mtime){
		// $is_staging_running = $this->config->get_option('is_staging_running');
		$new_file_size = @filesize($file_path);

		// if ($is_staging_running) {
		// 	if ($new_file_size == $prev_file_size){
		// 		// dark_debug(array(), '---------Same size------------');
		// 		return true;
		// 	}
		// 	// dark_debug(array(), '---------diff size------------');
		// 	return false;
		// }

		$this_file_m_time = @filemtime($file_path);

		if (($new_file_size == $prev_file_size) && ($this_file_m_time == $prev_file_mtime)) {
			return true;
		}
		return false;
	}
}